; -*-Emacs-Lisp-*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File:         gnu-misc.el
; RCS:          $Header: gnu-misc.el,v 2.1 89/04/24 18:37:38 darrylo Exp $
; Description:  Miscellaneous functions gleaned from the info-gnu-emacs
;		mailing list.
; Authors:      Many (see comments)
; Created:      Sat Apr  8 14:37:53 1989
; Modified:     Sat Sep 22 15:10:27 1990 (Darryl Okahata)
; Language:     Emacs-Lisp
; Package:      N/A
;
; (C) Copyright 1989, 1990, 1991, Hewlett-Packard, all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From allegra!kautz@EDDIE.MIT.EDU  Sat Sep 12 09:36:48 1987
;; Received: from EDDIE.MIT.EDU by PREP.AI.MIT.EDU; Sat, 12 Sep 87 09:36:48 EDT
;; Message-Id: <8709121336.AA12269@prep.ai.mit.edu>
;; Received: by EDDIE.MIT.EDU with UUCP with smail2.3 with sendmail-5.45/4.7 id <AA25488@EDDIE.MIT.EDU>; Sat, 12 Sep 87 09:32:09 EDT
;; From: allegra!kautz@EDDIE.MIT.EDU
;; Date: Sat, 12 Sep 87 09:15:10 edt
;; To: info-gnu-emacs@prep.ai.mit.edu
;; Subject: better insert file function version 2
;; 
;; A couple of days ago I posted a "better insert file function".  I just
;; determined (blush) that it didn't always work as well as it might for
;; pathnames which include "~" (namely /usr/name would not match against
;; ~).  So this version fixes that difficulty.
;; 
;; ---- Henry Kautz
;; :uucp:	allegra!kautz		
;; :csnet:	kautz@allegra.att.com
;; :arpa:	kautz%allegra%btl.csnet@csnet-relay.arpa
;; :mail:  AT&T Bell Laboratories	
;; 	Room 3C-402A
;; 	600 Mountain Ave.	
;; 	Murray Hill, N.J. 07974	
;; :office phone: (201) 582-2815	
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun insert-file-name-tail (name) 
 "Prompts for a file or directory name and inserts that name after point.
 Does not include unnecessary directory path prefixes.
 The name may be non-existent.  Useful in Shell mode."  
   (interactive "FInsert file name: ")
   (if (string-match (concat "^" (expand-file-name default-directory))
	  (expand-file-name name))
      (setq name (substring (expand-file-name name) (match-end 0))))
   (insert name))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; From liberte@b.cs.uiuc.edu  Tue Oct 27 23:04:10 1987
;; Received: by PREP.AI.MIT.EDU; Tue, 27 Oct 87 23:04:10 EST
;; Received: by b.cs.uiuc.edu (UIUC-5.52/9.7)
;; 	id AA05213; Tue, 27 Oct 87 22:02:36 CST
;; Date: Tue, 27 Oct 87 22:02:36 CST
;; From: liberte@b.cs.uiuc.edu (Daniel LaLiberte)
;; Message-Id: <8710280402.AA05213@b.cs.uiuc.edu>
;; To: info-gnu-emacs@prep.ai.mit.edu
;; Subject: deep-copy-keymap
;; 
;; Here is a function to create a deep copy of a keymap.  As was noted earlier,
;; copy-keymap does a shallow copy of just the top level.  This does the
;; job recursively on all subkeymaps as well.  I've used it a bit, but it
;; is not thoroughly tested.  It is difficult to see if a copy is really
;; different.
;; 
;; dan
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun deep-copy-keymap (keymap)
  "Return a deep copy of KEYMAP.  That is, all levels are copied,
not just the top level."
  (if (not (keymapp keymap))
      keymap
    (cond

     ((listp keymap)
      (let ((new-keymap (copy-alist keymap)))
	(setq keymap (cdr new-keymap))
	(while keymap
	  (let ((binding (car keymap)))
	    (if (keymapp (cdr binding))
		(setcdr binding (deep-copy-keymap (cdr binding))))
	    )
	  (setq keymap (cdr keymap))
	  )
	new-keymap
	))

      ((vectorp keymap)
       (let ((i 0)
	     (n (length keymap))
	     (new-keymap (copy-sequence keymap)))
	 (while (< i n)
	   (if (keymapp (aref keymap i))
	       (aset new-keymap i (deep-copy-keymap (aref keymap i))))
	   (setq i (1+ i)))
	 new-keymap
	 )))))
		
	 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From think!compass!worley@eddie.mit.edu  Thu Oct 20 04:19:03 1988
;; Received: from EDDIE.MIT.EDU by prep.ai.mit.edu; Thu, 20 Oct 88 04:19:03 EST
;; Received: by EDDIE.MIT.EDU with UUCP with smail2.5 with sendmail-5.45/4.7 id <AA04659@EDDIE.MIT.EDU>; Thu, 20 Oct 88 05:18:32 EDT
;; Received: by news.think.com; Thu, 20 Oct 88 05:08:08 EDT
;; Received: from galaxy.compass.com by compass.com (3.2/SMI-3.2)
;; 	id AA22458; Mon, 17 Oct 88 15:23:19 EDT
;; Received: by galaxy.compass.com (3.2/SMI-3.2)
;; 	id AA04446; Mon, 17 Oct 88 15:23:14 EDT
;; Date: Mon, 17 Oct 88 15:23:14 EDT
;; From: think!compass!worley@eddie.mit.edu (Dale Worley)
;; Message-Id: <8810171923.AA04446@galaxy.compass.com>
;; To: info-gnu-emacs@eddie.mit.edu
;; Subject: "Improvements" to find-alternate-file
;; 
;; I find that most of the times that I want to use find-alternate-file
;; are when I have attempted to find a file in the wrong directory.
;; find-alternate-file is annoying, because I then have to type the
;; filename over again.  This change to find-alternate-file fixes that --
;; if you gave the right filename, type C-u C-x C-v and just give the
;; correct directory.  The current filename of the buffer will be
;; appended.
;; 
;; Dale Worley, Compass, Inc.                      mit-eddie!think!compass!worley
;; Seen in a net discussion:  "It took work to make tofu politically correct."
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun find-alternate-file (filename &optional prefix)
  "Find file FILENAME, select its buffer, kill previous buffer.
If the current buffer now contains an empty file that you just visited
\(presumably by mistake), use this command to visit the file you really want.
If second argument is non-nil (or prefix argument is given interactively),
the current filename is appended to FILENAME (with a / if necessary) before
it is selected.  If you visited a file but gave the wrong directory, use this
command with a prefix argument to visit the right file, but you only have to
give the correct directory"
  (interactive "FFind alternate file: \nP")
  (and (buffer-modified-p)
       (not buffer-read-only)
       (not (yes-or-no-p (format "Buffer %s is modified; kill anyway? "
				 (buffer-name))))
       (error "Aborted"))
  (if prefix
      (progn
	(if (not (string-equal (substring filename -1) "/"))
	    (setq filename (concat filename "/")))
	(setq filename (concat filename
			       (file-name-nondirectory buffer-file-name)))))
  (let ((obuf (current-buffer))
	(ofile buffer-file-name)
	(oname (buffer-name)))
    (rename-buffer " **lose**")
    (setq buffer-file-name nil)
    (unwind-protect
	(progn
	  (unlock-buffer)
	  (find-file filename))
      (cond ((eq obuf (current-buffer))
	     (setq buffer-file-name ofile)
	     (lock-buffer)
	     (rename-buffer oname))))
    (kill-buffer obuf)))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From think!compass!worley@eddie.mit.edu  Thu Oct 20 04:18:21 1988
;; Received: from EDDIE.MIT.EDU by prep.ai.mit.edu; Thu, 20 Oct 88 04:18:21 EST
;; Received: by EDDIE.MIT.EDU with UUCP with smail2.5 with sendmail-5.45/4.7 id <AA04635@EDDIE.MIT.EDU>; Thu, 20 Oct 88 05:17:47 EDT
;; Received: by news.think.com; Thu, 20 Oct 88 05:07:56 EDT
;; Received: from galaxy.compass.com by compass.com (3.2/SMI-3.2)
;; 	id AA22448; Mon, 17 Oct 88 15:21:51 EDT
;; Received: by galaxy.compass.com (3.2/SMI-3.2)
;; 	id AA04440; Mon, 17 Oct 88 15:21:47 EDT
;; Date: Mon, 17 Oct 88 15:21:47 EDT
;; From: think!compass!worley@eddie.mit.edu (Dale Worley)
;; Message-Id: <8810171921.AA04440@galaxy.compass.com>
;; To: info-gnu-emacs@eddie.mit.edu
;; Subject: Function for saving the results of a help request
;; 
;; Sometimes you want to save the *Help* buffer for later examination,
;; e.g., when you do an apropos.  save-help will rename the *Help* buffer
;; *Help<1>*, *Help<2>*, etc., so the information won't get clobbered by
;; further help requests.
;; 
;; Dale Worley, Compass, Inc.                      mit-eddie!think!compass!worley
;; Seen in a net discussion:  "It took work to make tofu politically correct."
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun save-help ()
  (interactive)
  (save-excursion
    (let ((i 1) 
	  (buffer (get-buffer "*Help*"))
	  name)
      (if (not buffer)
	  (ding)
	(while
	    (progn
	      (setq name (concat "*Help<" (int-to-string i) ">*"))
	      (get-buffer name))
	  (setq i (1+ i)))
	(set-buffer buffer)
	(rename-buffer name)
	(message (concat "Help buffer renamed " name))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From info-gnu-emacs-request@prep.ai.mit.edu  Mon Oct 24 19:51:40 1988
;; Received: from TUT.CIS.OHIO-STATE.EDU by prep.ai.mit.edu; Mon, 24 Oct 88 19:51:40 EST
;; Received: by tut.cis.ohio-state.edu (5.54/2.880920)
;; 	id AA24449; Mon, 24 Oct 88 20:42:46 EDT
;; Received: from USENET by tut.cis.ohio-state.edu with netnews
;; 	for info-gnu-emacs@prep.ai.mit.edu (info-gnu-emacs@prep.ai.mit.edu)
;; 	(contact usenet@tut.cis.ohio-state.edu if you have questions)
;; Date: 24 Oct 88 22:51:01 GMT
;; From: amdahl!daveemme@AMES.ARC.NASA.GOV  (Dave Emme)
;; Organization: Amdahl Corporation, Sunnyvale CA
;; Subject: A find-file Hack
;; Message-Id: <c7XP=dcODs1010V13j.@amdahl.uts.amdahl.com>
;; Sender: info-gnu-emacs-request@prep.ai.mit.edu
;; To: info-gnu-emacs@prep.ai.mit.edu
;; 
;; 
;; Here's a little bit of code to be run as a "find-file-hook".  It runs the
;; UNIX command "ls -l" on the filename being visited and places the result in
;; the message line.  This gives you the file size, date last modified,
;; file permissions, etc. automatically.
;; 
;; Enjoy.
;; 
;; -- Dave
;; 	(Insert disclaimer of your choice here)
;; 
;; Dave Emme  (M/S 316)                    daveemme@uts.amdahl.com
;; Amdahl Corporation            {ames, decwrl, sun, uunet}!amdahl!daveemme
;; 1250 E. Arques Ave.
;; Sunnyvale, CA 94088
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Place file information in the message line when a file is visited.

(defvar display-file-info-buffer "*display-file-info-buffer*")
 
(defun display-file-info ()
  "Run `ls -l <filename>' whenever a file is visited, placing the
output in the message line."
  (let ((temp-buffer-show-hook 'display-file-info-hook)
	(file (cdr (assq 'buffer-file-name (buffer-local-variables)))))
    (with-output-to-temp-buffer display-file-info-buffer
      (call-process "/bin/ls" nil standard-output nil
		    "-l" (expand-file-name file))
      )))

(defun display-file-info-hook (buffer)
  ;; This hook prevents the temporary buffer created by display-file-info
  ;; from being displayed.  Instead it `messages' the information.
  (save-excursion
    (set-buffer display-file-info-buffer)
    (let ((str (buffer-substring (point-min) (1- (point-max)))))
      (message str))
    (kill-buffer buffer)))

;;
;; Execute the next two lines to run display-file-info each time a file is
;; visited.
;;
;;(if (not (memq 'display-file-info find-file-hooks))
;;    (setq find-file-hooks (cons 'display-file-info find-file-hooks)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From info-gnu-emacs-request@prep.ai.mit.edu  Wed Oct 26 01:20:50 1988
;; Received: from TUT.CIS.OHIO-STATE.EDU by prep.ai.mit.edu; Wed, 26 Oct 88 01:20:50 EST
;; Received: by tut.cis.ohio-state.edu (5.59/2.880920)
;; 	id AA14845; Wed, 26 Oct 88 02:00:20 EDT
;; Received: from USENET by tut.cis.ohio-state.edu with netnews
;; 	for info-gnu-emacs@prep.ai.mit.edu (info-gnu-emacs@prep.ai.mit.edu)
;; 	(contact usenet@tut.cis.ohio-state.edu if you have questions)
;; Date: 25 Oct 88 20:59:18 GMT
;; From: birdland!rberlin@sun.com  (Rich Berlin)
;; Subject: Re: A find-file Hack
;; Message-Id: <74562@sun.uucp>
;; References: <8810251636.AA10923@ELI.CS.YALE.EDU>
;; Sender: info-gnu-emacs-request@prep.ai.mit.edu
;; To: info-gnu-emacs@prep.ai.mit.edu
;; 
;; 
;; To simulate ls -l using file-attributes is certainly possible, but
;; since there doesn't seem to be an elisp-interface to getpwent, you
;; have to search /etc/passwd for the uid number (and punt or do
;; something else ugly if you're using a name server like NFS, where the
;; entry may not be in /etc/passwd....)  On my machine (a Sun 3/160,
;; 12MB, running 4.0), using /bin/ls is acceptably fast and a lot easier.
;; 
;; But, now I am thinking about trying to support the getpwent function.
;; Opinions (especially from RMS) about the practice of adding interfaces
;; to system calls?  It certainly raises the risk of writing
;; system-specific emacs extensions!
;; 
;; 
;; By the way, I've had other occasions when I wanted the output of a command,
;; so I wrote a function which, rather than applying 'message to the buffer
;; contents, returns them as a string.  It's easy to then pass the string to
;; message:
;; 
;; -- Rich
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun results-of-command (command &rest args)
  "Execute the given COMMAND with &rest ARGS, and return the resulting\n\
output as a string."
  (let (foo
	(temp-buffer-show-hook
	 '(lambda (buffer)
	    (set-buffer buffer)
	    (setq foo (buffer-substring (point-min) (1- (point-max))))
	    (kill-buffer buffer))))
    (save-excursion
      (with-output-to-temp-buffer "*results of command*"
	(apply 'call-process command nil standard-output nil args))
    foo)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; From mcvax!moncam!paul@uunet.uu.net  Fri Nov 18 18:08:22 1988
;; Received: from [192.12.141.129] by prep.ai.mit.edu; Fri, 18 Nov 88 18:08:22 EST
;; Received: from mcvax.UUCP by uunet.UU.NET (5.59/1.14) with UUCP 
;; 	id AA21262; Fri, 18 Nov 88 18:12:57 EST
;; Received: by mcvax.cwi.nl via EUnet; Sat, 19 Nov 88 00:03:37 +0100 (MET)
;; Received: from acorn by kestrel.Ukc.AC.UK   with UUCP  id aa14271;
;;           18 Nov 88 12:08 GMT
;; Received: by acorn.co.uk (5.51/AM10)
;; 	id AA19721; Fri, 18 Nov 88 11:03:26 GMT
;; Received: by marvin.moncam.uucp (3.2/SMI-3.2)
;; 	id AA09267; Fri, 18 Nov 88 10:55:11 GMT
;; Date: Fri, 18 Nov 88 10:55:11 GMT
;; From: "Paul (Should a pun..." <mcvax!moncam!paul@uunet.uu.net>
;; Mmdf-Warning:  Parse error in original version of preceding line at Ukc.AC.UK
;; Message-Id: <8811181055.AA09267@marvin.moncam.uucp>
;; To: info-gnu-emacs@prep.ai.mit.edu
;; Subject: Fn. to line up assigment ops.
;; 
;; 
;; The following little bit of lisp will ensure the first assignment operators
;; on each of the lines line up. This is part of our local formatting style
;; 'cos it looks nice ;-)
;; 
;; The style of the lisp however, is atrocious. All the problems come from ==,
;; which looks too much like 'op='.
;; 
;; Enjoy.
;; 
;; Paul Hudson 
;; 
;; Snail mail: Monotype ADG	Email:	...!ukc!acorn!moncam!paul
;; 	    Science Park,		paul@moncam.co.uk
;; 	    Milton Road,	"Sun Microsysytems:
;; 	    Cambridge,		 The Company is Arrogant (TM)"
;; 	    CB4 4FQ
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun align-equals (start end)
 "make the first assignment operator on each line line up vertically"
 (interactive "*r")
 (save-excursion
   (let ((indent 0))
     (narrow-to-region start end)
     (beginning-of-buffer)
     (while (not (eobp))
       (if (find-assignment)
	   (progn
	     (exchange-point-and-mark)
	     (setq indent (max indent (current-column)))
	     (delete-horizontal-space)
	     (insert " ")))
       (forward-line 1))
     (beginning-of-buffer)
     (while (not (eobp))
       (if (find-assignment)
	   (indent-to-column (1+ (- indent  (- (mark) (point))))))
       (forward-line 1)))
   (widen)))


(defun find-assignment ()
  (if (re-search-forward
	     "[^<>=!]=\\|\\+=\\|-=\\|\\*=\\|/=\\|&=\\||=\\|\\^=\\|<<=\\|>>="
	     (save-excursion (end-of-line) (point)) t)
      (progn
	(goto-char (match-beginning 0))
	(if (looking-at ".==")
	    nil
	  (if (looking-at "\\+=\\|-=\\|\\*=\\|/=\\|&=\\||=\\|\\^=\\|<<=\\|>>=")
	      (set-mark (match-end 0))
	    (forward-char 1)
	    (set-mark (1+ (point))))
	  (delete-horizontal-space)
	  t))
    nil))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Date: 20 Aug 90 16:09:20 GMT
;; From: news@iwarp.intel.com  (Randal Schwartz)
;; Organization: Stonehenge; netaccess via Intel, Beaverton, Oregon, USA
;; Subject: Re: Shell mode with more than one shell?
;; Sender: info-gnu-emacs-request@prep.ai.mit.edu
;; To: info-gnu-emacs@prep.ai.mit.edu
;; 
;; In article <1990Aug20.075002.12385@solar.card.inpu.oz.au>, brett@solar (Brett Sealey) writes:
;; | I asked about how to get more than one inferior shell process under emacs.
;; | 
;; | My thanks to Carl Witty for his prompt reply.
;; | 
;; | > If you rename the shell buffer yourself, the next time you type M-x
;; | > shell you get a "new" shell.
;; | > 
;; | > (i.e. M-x rename-buffer RET *shell-2* RET)
;; | 
;; | This was just what the doctor ordered.
;; | 
;; | Thanks.
;; 
;; I never use shell-mode, preferring it instead to this package that I
;; shamelessly stole the idea from someone whose name escapes me now.
;; 
;; For an interactive shell, invoke a command of "sh".  But typically,
;; it's easier to invoke the command you want directly.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; clip here	
;;; original version by merlyn
;;; LastEditDate="Thu Mar 16 13:16:13 1989"

(defun background-shell (command &optional show)
  "Create a background shell on COMMAND.
If optional SHOW is non-nil (prefix argument if interactive), force
the buffer for COMMAND to be visible in a window."
  (interactive "s& \nP")
  (let ((process (start-process "&shell"
				(generate-new-buffer "&shell")
				shell-file-name
				"-c"
				command)))
    (set-marker (process-mark process) 1 (process-buffer process))
    (set-process-filter
     process
     (function
      (lambda (process text)
	(cond ((buffer-name (process-buffer process))
	       (save-excursion
		 (set-buffer (process-buffer process))
		 (goto-char (process-mark process))
		 (let ((bmp (buffer-modified-p)))
		   (insert-before-markers text)
		   (set-buffer-modified-p bmp))))))))
    (set-process-sentinel
     process
     (function
      (lambda (process reason)
	(setq reason (substring reason 0 -1)) ; kill \n at end
	(let ((status (process-status process)))
	  (cond ((memq status '(stop signal exit))
		 (cond ((buffer-name (process-buffer process))
			(save-excursion
			  (set-buffer (process-buffer process))
			  (goto-char (point-max)) ; should be process-mark?
			  (let ((bmp (buffer-modified-p)))
			    (insert-before-markers
			     "\n\n*** "
			     (current-time-string) ": " reason
			     " ***\n\n")
			    (set-buffer-modified-p bmp))
			  (let ((cw (get-buffer-window (current-buffer))))
			    (if cw
				(set-window-start
				 cw
				 (save-excursion
				   (forward-line (- 2 (window-height cw)))
				   (point))
				 t)))
			    
			  (cond ((memq status '(signal exit))
				 (setq mode-line-process nil))))))
		 (message "[process %s in buffer %s %s]"
			  (process-name process)
			  (buffer-name (process-buffer process))
			  reason
			  (ding t))))))))
    (save-excursion
      (set-buffer (process-buffer process))
      (require 'shell)
      (shell-mode)
      (insert-before-markers "*** Input ***\n" command "\n*** Output ***\n")
      (setq mode-line-process '(": %s"))
      (set-buffer-modified-p nil)
      (if show
	  (let ((sw (selected-window)))
	    (select-window (display-buffer (current-buffer) t))
	    (select-window sw))))
    (message "[started process %s in buffer %s]"
	     (process-name process)
	     (buffer-name (process-buffer process)))))

